@@ -4,29 +4,19 @@
  * software, which is freely distributable under the terms of the
  * GNU public license, included as the file COPYING in this
  * distribution.  It is NOT public domain software, and any
- * redistribution not permitted by the GNU Public License is
+ * redistribution not permitted by the GNU General Public License is
  * expressly forbidden without prior written permission from
  * the author.
  *
  */
 
-/* RCS log:
- * $Log: help.c,v $
- * Revision 1.2  1994/03/23 17:00:16  johnsonm
- * Removed reference to --pattern and -p.
- *
- * Revision 1.1  1994/03/13  16:28:16  johnsonm
- * Initial revision
- *
- */
-
 
 #include <stdio.h>
 #include <unistd.h>
 #include "vlock.h"
 
 
-static char rcsid[] = "$Id: help.c,v 1.2 1994/03/23 17:00:16 johnsonm Exp johnsonm $";
+static char rcsid[] = "$Id: help.c,v 1.3 1997/10/10 17:08:14 johnsonm Exp $";
 
 
 void print_help(int exitcode) {
