@@ -4,70 +4,12 @@
  * software which is freely distributable under the terms of the
  * GNU public license, included as the file COPYING in this
  * distribution.  It is NOT public domain software, and any
- * redistribution not permitted by the GNU Public License is
+ * redistribution not permitted by the GNU General Public License is
  * expressly forbidden without prior written permission from
  * the author.
  *
  */
 
-/* RCS log:
- * $Log: input.c,v $
- * Revision 1.15  1996/06/21 23:27:17  johnsonm
- * PAM support
- *
- * Revision 1.14  1996/05/17 02:49:46  johnsonm
- * Added Marek's changes, which add proper shadow support, and fix
- * a few other niggling bugs.
- *
- * Revision 1.13  1994/07/03 13:07:47  johnsonm
- * Set and restore signals and terminal settings correctly.
- *
- * Revision 1.12  1994/07/03  12:29:27  johnsonm
- * Don't restore terminal, which restores signals, which makes vlock -a
- *   completely ineffective, until the correct password has been entered.
- *
- * Revision 1.11  1994/07/03  12:19:29  johnsonm
- * Moved shadow comment to first SHADOW define.
- *
- * Revision 1.10  1994/07/03  12:14:49  johnsonm
- * Restores signal mask after getting password
- *
- * Revision 1.9  1994/07/03  12:11:24  johnsonm
- * Updated shadow comment to explain.
- *
- * Revision 1.8  1994/03/23  17:00:01  johnsonm
- * Control-function -> Alt-function
- * Added support for non-vt consoles.
- *
- * Revision 1.7  1994/03/21  17:33:33  johnsonm
- * Reformatted "This TTY is now locked" message to read easier.
- *
- * Revision 1.6  1994/03/21  17:30:55  johnsonm
- * Allow root to lock console.  Was only disallowed because I took
- * the correct_password function from gnu su, where of course root
- * automatically succeeds at entering the password without haveing
- * to enter it.  Oops.
- *
- * Revision 1.5  1994/03/20  13:13:48  johnsonm
- * Added code so that root password unlocks the session too.
- *
- * Revision 1.4  1994/03/19  17:53:20  johnsonm
- * Password reading now works, thanks to the correct_password() routine
- * from GNU su.  No point in re-inventing the wheel...
- *
- * Revision 1.3  1994/03/19  14:24:33  johnsonm
- * Removed silly two-process model.  It's certainly not needed.
- * Also fixed occasional core dump.
- *
- * Revision 1.2  1994/03/16  20:12:06  johnsonm
- * Now almost working.  Need to get signals straightened out for
- * second process.
- *
- * Revision 1.1  1994/03/15  18:27:33  johnsonm
- * Initial revision
- *
- */
-
   /* Marek hacked the code a bit:
       - use /dev/tty instead of /dev/console (permissions on the latter
         shouldn't allow all users to open it)
@@ -84,10 +26,12 @@
         if getpass() fails.
   */
 
+#include <errno.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <signal.h>
+#include <sys/types.h>
 #include <pwd.h>
 #ifdef SHADOW_PWD
   /* Shadow passwd support; THIS IS NOT SAFE with some very old versions
@@ -116,58 +60,10 @@
 #endif
 
 #include <security/pam_appl.h>
-/* Static variables used to communicate between the conversation function
- * and the server_login function
- */
-static char *PAM_username;
-static char *PAM_password;
+#include <security/pam_misc.h>
 
-/* PAM conversation function
- * Here we assume (for now, at least) that echo on means login name, and
- * echo off means password.
- */
-static int PAM_conv (int num_msg,
-                     const struct pam_message **msg,
-		     struct pam_response **resp,
-		     void *appdata_ptr) {
-  int count = 0, replies = 0;
-  struct pam_response *reply = NULL;
-  int size = sizeof(struct pam_response);
-
-  #define GET_MEM if (reply) realloc(reply, size); else reply = malloc(size); \
-  if (!reply) return PAM_CONV_ERR; \
-  size += sizeof(struct pam_response)
-  #define COPY_STRING(s) (s) ? strdup(s) : NULL
-
-  for (count = 0; count < num_msg; count++) {
-    switch (msg[count]->msg_style) {
-      case PAM_PROMPT_ECHO_ON:
-        GET_MEM;
-        reply[replies].resp_retcode = PAM_SUCCESS;
-	reply[replies++].resp = COPY_STRING(PAM_username);
-          /* PAM frees resp */
-        break;
-      case PAM_PROMPT_ECHO_OFF:
-        GET_MEM;
-        reply[replies].resp_retcode = PAM_SUCCESS;
-	reply[replies++].resp = COPY_STRING(PAM_password);
-          /* PAM frees resp */
-        break;
-      case PAM_TEXT_INFO:
-        /* ignore it... */
-        break;
-      case PAM_ERROR_MSG:
-      default:
-        /* Must be an error of some sort... */
-        free (reply);
-        return PAM_CONV_ERR;
-    }
-  }
-  if (reply) *resp = reply;
-  return PAM_SUCCESS;
-}
 static struct pam_conv PAM_conversation = {
-    &PAM_conv,
+    &misc_conv,
     NULL
 };
 
@@ -175,11 +71,11 @@ static struct pam_conv PAM_conversation = {
 #include "vlock.h"
 
 
-static char rcsid[] = "$Id: input.c,v 1.15 1996/06/21 23:27:17 johnsonm Exp johnsonm $";
+static char rcsid[] = "$Id: input.c,v 1.23 1998/03/12 13:11:04 johnsonm Exp $";
 
 
-static char username[40]; /* current user's name */
 static char prompt[100];  /* password prompt ("user's password: ") */
+static char username[40]; /* current user's name */
 #ifndef USE_PAM
 static char userpw[200];  /* current user's encrypted password */
 #ifndef NO_ROOT_PASS
@@ -192,11 +88,50 @@ static char rootpw[200];  /* root's encrypted password */
 static int
 correct_password(void)
 {
-  char *pass;
 #ifdef USE_PAM
   pam_handle_t *pamh;
   int pam_error;
-#endif
+
+  /* Now use PAM to do authentication.
+   */
+  #define PAM_BAIL if (pam_error != PAM_SUCCESS) { \
+     pam_end(pamh, 0); \
+     /* fix signals that may have been disordered by pam */ \
+     set_signal_mask(0); \
+     return 0; \
+     }
+  pam_error = pam_start("vlock", username, &PAM_conversation, &pamh);
+  PAM_BAIL;
+  printf("%s's ", username); fflush(stdout);
+  pam_error = pam_set_item(pamh, PAM_USER_PROMPT, strdup(prompt));
+  PAM_BAIL;
+  pam_error = pam_authenticate(pamh, 0);
+  /* fix signals that may have been disordered by pam */
+  set_signal_mask(0);
+#ifdef NO_ROOT_PASS
+  PAM_BAIL;
+#else
+  if (pam_error != PAM_SUCCESS) {
+    /* Try as root; bail if no success there either */
+    printf("root's "); fflush(stdout);
+    pam_error = pam_set_item(pamh, PAM_USER_PROMPT, strdup(prompt));
+    PAM_BAIL;
+    pam_error = pam_set_item(pamh, PAM_USER, "root");
+    PAM_BAIL;
+    pam_error = pam_authenticate(pamh, 0);
+    /* fix signals that may have been disordered by pam */
+    set_signal_mask(0);
+    PAM_BAIL;
+  }
+#endif /* !NO_ROOT_PASS */
+  pam_end(pamh, PAM_SUCCESS);
+  /* If this point is reached, the user has been authenticated. */
+  return 1;
+
+
+
+#else /* !PAM */
+  char *pass;
 
   pass = getpass(prompt);
   if (!pass) {
@@ -204,39 +139,7 @@ correct_password(void)
     restore_terminal();
     exit(1);
   }
-  /* fix signals that probably have been disordered by getpass() */
-  set_signal_mask(0);
 
-#ifdef USE_PAM
-  /* Now use PAM to do authentication.
-   */
-  #define PAM_BAIL if (pam_error != PAM_SUCCESS) { \
-     pam_end(pamh, 0); \
-     memset(pass, 0, strlen(pass)); \
-     return 0; \
-     }
-  PAM_password = pass;
-  PAM_username = username;
-  pam_error = pam_start("vlock", username, &PAM_conversation, &pamh);
-  PAM_BAIL;
-  pam_error = pam_authenticate(pamh, 0);
-#ifdef NO_ROOT_PASS
-  PAM_BAIL;
-#else
-  if (pam_error != PAM_SUCCESS) {
-    /* Try as root; bail if no success there either */
-    pam_error = pam_set_item(pamh, PAM_USER, "root");
-    PAM_BAIL;
-    pam_error = pam_authenticate(pamh, 0);
-    PAM_BAIL;
-  }
-#endif /* !NO_ROOT_PASS */
-  pam_end(pamh, PAM_SUCCESS);
-  /* If this point is reached, the user has been authenticated. */
-  memset(pass, 0, strlen(pass));
-  return 1;
-
-#else /* !PAM */
   if (strcmp(crypt(pass, userpw), userpw) == 0) {
     memset(pass, 0, strlen(pass));
     return 1;
@@ -248,9 +151,9 @@ correct_password(void)
     return 1;
   }
 #endif
-#endif /* !USE_PAM */
-
   memset(pass, 0, strlen(pass));
+
+#endif /* !USE_PAM */
   return 0;
 }
 
@@ -284,18 +187,34 @@ get_password(void)
       restore_terminal();
       return;
     }
+
+    /* correct_password() may have failed to return success because the   */
+    /* terminal is closed.  In this case, it would not be appropriate to  */
+    /* try again...                                                       */
+    if (isatty(STDIN_FILENO) == 0) {
+	perror("isatty");
+	restore_terminal();
+	exit(1);
+    }
+
+
     /* Need to slow down people who are trying to break in by brute force */
     /* Note that it is technically possible to break this, but I can't    */
     /* make it happen, even knowing the code and knowing how to try.      */
     /* If you manage to kill vlock from the terminal while in this code,  */
     /* please tell me how you did it.                                     */
+#ifndef USE_PAM
+    /* This is policy; when we use PAM, we should let it determine policy */
     sleep(++times);
+#endif
     printf(" *** That password is incorrect; please try again. *** \n");
+#ifndef USE_PAM
     if (times >= 15) {
       printf("Slow down and try again in a while.\n");
       sleep(times);
       times = 2; /* don't make things too easy for someone to break in */
     }
+#endif
     printf("\n");
 
   } while (1);
@@ -327,7 +246,7 @@ my_getpwuid(uid_t uid)
 
   pw = getpwuid(uid);
   if (!pw) {
-    fprintf(stderr, "vlock: getpwuid(%d) failed!\n", (int) uid);
+    fprintf(stderr, "vlock: getpwuid(%d) failed: %s\n", uid, strerror(errno));
     exit(1);
   }
 #ifdef SHADOW_PWD
@@ -372,8 +291,8 @@ init_passwords(void)
 
   /* We don't need root privileges any longer.  */
   setuid(getuid());
-#endif /* !USE_PAM */
 
   sprintf(prompt, "%s's password: ", username);
+#endif /* !USE_PAM */
 }
 
